<?php
/*
Plugin Name: WP htaccess Control
Plugin URI: http://dardna.com/wp-htaccess-control
Description: Provides an interface to add custom htaccess rules to the htaccess file generated by wordpress. Also provides customisation of the Author Permalink Base and of Paginated Permalinks.
Version: 1.5.3
Author: António Andrade
Author URI: http://dardna.com
*/
/*  Copyright 2010  António Andrade  (email : dardna@dardna.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/
if (!class_exists("WPhtc")) {
	class WPhtc {
		function WPhtc() { 
			}
		function wphtc_init(){
			$currentLocale = get_locale();
			if(!empty($currentLocale)) {
				$moFile = dirname(__FILE__) . "/lang/wp-htaccess-control-" . $currentLocale . ".mo";
				if(@file_exists($moFile) && is_readable($moFile)) load_textdomain('wp-htaccess-control', $moFile);
				}
			}
		# Filter pagination links generated by Wordpress
		function wphtc_filter_get_pagenum_link($result){
			$WPhtc_data=get_option('WPhtc_data');
			if(isset($WPhtc_data['cpp'])&&$WPhtc_data['cpp']!=''){
				$result=preg_replace("/".$WPhtc_data['cpp']."\/[0-9]+[\/]?/","",$result);
				$result=str_replace("page",$WPhtc_data['cpp'],$result);
				}
			return $result;
			}
		# Filter link generated by get_author_posts_url() to use the Custom Author Permalink
		function wphtc_filter_author_link($link){
			$WPhtc_data=get_option('WPhtc_data');
			if(isset($WPhtc_data['cap']) && $WPhtc_data['cap']!=''){
				$link=str_replace("author",$WPhtc_data['cap'],$link);
				}
			return $link;
			}
		# Disable canonical redirection on urls using custom pagination permalink
		function wphtc_filter_redirect_canonical($redirect_url,$requested_url){
			$WPhtc_data=get_option('WPhtc_data');
			if(isset($WPhtc_data['cpp'])&&$WPhtc_data['cpp']!=''&&get_query_var('paged') > 1&&strpos($requested_url,$WPhtc_data['cpp'])){
				return false;
				}
			else{
				return $redirect_url;
				}
			}
		# Replacing the Wordpress Author Permalink by rewriting its rules if we have a Custom Author Permalink, everytime the rules are flush 
		function wphtc_filter_rewrite_rules($rules){
			$WPhtc_data=get_option('WPhtc_data');
			if((isset($WPhtc_data['cap'])&&$WPhtc_data['cap']!='')||(isset($WPhtc_data['cpp'])&&$WPhtc_data['cpp']!='')){
				$rules=serialize($rules);
				if(isset($WPhtc_data['cap'])&&$WPhtc_data['cap']!=''){
					$rules=str_replace('author/',$WPhtc_data['cap'].'/',$rules);
					}
				if(isset($WPhtc_data['cpp'])&&$WPhtc_data['cpp']!=''){
					$rules=str_replace('page/',$WPhtc_data['cpp'].'/',$rules);
					}
				$rules=unserialize(preg_replace('!s:(\d+):"(.*?)";!se', "'s:'.strlen('$2').':\"$2\";'",$rules));
				}
			return $rules;
			}
		# Adding our Custom Author Base to the sitemap generated by Google XML Sitemaps
		# Adapted from sitemap-core.php line 2036
		function set_sm(){
			$WPhtc_data=get_option('WPhtc_data');
			if(class_exists('GoogleSitemapGenerator')&&$WPhtc_data['cap']!=''){
				$generatorObject = &GoogleSitemapGenerator::GetInstance();
				if ($generatorObject != null){
					global $wpdb;
					$sql = "SELECT DISTINCT
							p.ID,
							u.user_nicename,
							MAX(p.post_modified_gmt) AS last_post
						FROM
							{$wpdb->users} u,
							{$wpdb->posts} p
						WHERE
							p.post_author = u.ID
							AND p.post_status = 'publish'
							AND p.post_type = 'post'
							AND p.post_password = ''
							" . (floatval($wp_version) < 2.1?"AND p.post_date_gmt <= '" . gmdate('Y-m-d H:i:59') . "'":"") . "
						GROUP BY
							u.ID,
							u.user_nicename";
					$authors = $wpdb->get_results($sql);	
					if($authors && is_array($authors)) {
						foreach($authors as $author) {
							$url=get_bloginfo('home')."/".$WPhtc_data['cap']."/".$author->user_nicename."/";
							$generatorObject->AddUrl($url,$generatorObject->GetTimestampFromMySql($author->last_post),$generatorObject->GetOption("cf_auth"),$generatorObject->GetOption("pr_auth"));
							}
						}
					}
				}
			}
		# Adding rewrite pairs to the .htaccess generated by Wordpress, everytime the rules are flush
		function wphtc_wp_rewrite_rules($wp_rewrite){
			$WPhtc_data=get_option('WPhtc_data');
			for($i=0;$i<count($WPhtc_data['pats']);$i++){
				if($WPhtc_data['pats'][$i]!=''&&$WPhtc_data['subs'][$i]!=''){
					# Adding each custom rewrite pair to $wp_rewrite->non_wp_rules['pattern']='substitution'
					$wp_rewrite->non_wp_rules[$WPhtc_data['pats'][$i]]=$WPhtc_data['subs'][$i];
					}
				}
			# Keeping a copy of the generated htaccess in our option array for later reviewing
			$WPhtc_data['cur_hta']='# BEGIN Wordpress<br/>'.nl2br($wp_rewrite->mod_rewrite_rules()).'# END Wordpress';
			update_option('WPhtc_data',$WPhtc_data);
			return $wp_rewrite;
			}
		# Adding Custom Rules to htaccess file generated by Wordpress, everytime the rules are flush
		function wphtc_hta_rules($rules){
			$WPhtc_data=get_option('WPhtc_data');
			if($WPhtc_data['hta']){
				$new_rules.="\n# WPhtc: Begin Custom htaccess\n";
				$new_rules.=stripslashes($WPhtc_data['hta'])."\n";
				$new_rules.="# WPhtc: End Custom htaccess\n";
				}
			# htaccess suggestions
			if($WPhtc_data['disable_serversignature']){
				$new_rules.="\n# WPhtC: Disable ServerSignature on generated error pages\n";
				$new_rules.="ServerSignature Off\n";
				}
			if($WPhtc_data['admin_email']){
				$new_rules.="\n# WPhtC: Set admin email\n";
				$new_rules.="SetEnv SERVER_ADMIN ".$WPhtc_data['admin_email']."\n\n";
				}
			if($WPhtc_data['disable_indexes']){
				$new_rules.="\n# WPhtC: Disable directory browsing\n";
				$new_rules.="Options All -Indexes\n";
				}
			if($WPhtc_data['up_limit']){
				$new_rules.="\n# WPhtC: Limit upload size to ".$WPhtc_data['up_limit']." MB\n";
				$new_rules.="LimitRequestBody ".($WPhtc_data['up_limit']*1024*1024)." \n";
				}
			if($WPhtc_data['redirect_500']){
				$new_rules.="\n# WPhtC: Setting 500 Error page\n";
				$new_rules.="ErrorDocument 500 ".$WPhtc_data['redirect_500']."\n";
				}
			if($WPhtc_data['protect_wp_config']){
				$new_rules.="\n# WPhtC: Protect WP-config.php\n";
				$new_rules.="<files wp-config.php>\n";
				$new_rules.="order allow,deny\n";
				$new_rules.="deny from all\n";
				$new_rules.="</files>\n";
				}
			if($WPhtc_data['protect_htaccess']){
				$new_rules.="\n# WPhtC: Protect .htaccess file\n";
				$new_rules.="<files ~ \"^.*\.([Hh][Tt][Aa])\">\n";
				$new_rules.="order allow,deny\n";
				$new_rules.="deny from all\n";
				$new_rules.="</files>\n";
				}
			if($WPhtc_data['disable_hotlink']){
				$new_rules.="\n# WPhtC: Disable image hotlinking\n";
				$new_rules.="<IfModule mod_rewrite.c>\n";
				$new_rules.="RewriteEngine On\n";
				$new_rules.="RewriteCond %{HTTP_REFERER} !^$\n";
				$new_rules.="RewriteCond %{HTTP_REFERER} !^http://(www\.)?".str_ireplace(array("http://","www."),"",get_bloginfo("url"))."/.*$ [NC]\n";
				$new_rules.="RewriteRule \.(jpg|jpeg|png|gif)$ ".$WPhtc_data['disable_hotlink']." [NC,R,L]\n";
				$new_rules.="</IfModule>\n";
				}
			if($WPhtc_data['canon']){
				$url=str_ireplace(array("http://","www."),"",get_bloginfo("url"));
				$domain=explode("/",$url);
				$escaped_domain=str_ireplace(".","\.",$domain[0]);
				if($WPhtc_data['canon']=='www'){
					$new_rules.="\n# WPhtC: Setting WWW canon\n";
					$new_rules.="<IfModule mod_rewrite.c>\n";
					$new_rules.="RewriteEngine On\n";
					$new_rules.="RewriteCond %{HTTP_HOST} ^".$escaped_domain." [NC] \n";
					$new_rules.="RewriteRule (.*) http://www.".$domain[0]."/$1 [R=301,L]\n";
					$new_rules.="</IfModule>\n";
					}
				else if($WPhtc_data['canon']=='simple'){
					$new_rules.="\n# WPhtC: Setting canon to non-WWW\n";
					$new_rules.="<IfModule mod_rewrite.c>\n";
					$new_rules.="RewriteEngine On\n";
					$new_rules.="RewriteCond %{HTTP_HOST} ^www\.".$escaped_domain." [NC] \n";
					$new_rules.="RewriteRule (.*) http://".$domain[0]."/$1 [R=301,L]\n";
					$new_rules.="</IfModule>\n";
					}
				}
			if($WPhtc_data['gzip']){
					$new_rules.="\n# WPhtC: Setting mod_gzip\n";
					$new_rules.="<ifModule mod_gzip.c>\n";
					$new_rules.="mod_gzip_on Yes\n";
					$new_rules.="mod_gzip_dechunk Yes\n";
					$new_rules.="mod_gzip_item_include file \.(html?|txt|css|js|php|pl)$\n";
					$new_rules.="mod_gzip_item_include handler ^cgi-script$\n";
					$new_rules.="mod_gzip_item_include mime ^text/.*\n";
					$new_rules.="mod_gzip_item_include mime ^application/x-javascript.*\n";
					$new_rules.="mod_gzip_item_exclude mime ^image/.*\n";
					$new_rules.="mod_gzip_item_exclude rspheader ^Content-Encoding:.*gzip.*\n";
					$new_rules.="</ifModule>\n";
				}
			if($WPhtc_data['deflate']){
					$new_rules.="\n# WPhtC: Setting mod_deflate\n";
					$new_rules.="<IfModule mod_deflate.c>\n";
					$new_rules.="AddOutputFilterByType DEFLATE text/html text/plain text/xml application/xml application/xhtml+xml text/javascript text/css application/x-javascript\n";	
					$new_rules.="BrowserMatch ^Mozilla/4 gzip-only-text/html\n";
					$new_rules.="BrowserMatch ^Mozilla/4.0[678] no-gzip\n";
					$new_rules.="BrowserMatch bMSIE !no-gzip !gzip-only-text/html\n";
					$new_rules.="Header append Vary User-Agent env=!dont-vary\n";
					$new_rules.="</IfModule>\n";
				}
			$new_rules.="\n".$rules;
			return $new_rules;
			}
		# WP-htaccess Control options page interface
		function wphtc_page(){
			include (dirname (__FILE__).'/wp-htaccess-control-ui.php');
			}
		# Adding our options page to the admin menu
		function configure_menu(){
			if(current_user_can("administrator")){
				$page=add_submenu_page("options-general.php","WP htaccess Control", "WP htaControl", 6, __FILE__, array('WPhtc','wphtc_page'));
				add_action('admin_print_scripts-'.$page, array('WPhtc','wphtc_page_script'));
				add_action('admin_print_styles-'.$page, array('WPhtc','wphtc_page_style'));
				}
			}
		# Enqueue Ui Scripts on Plugin page
		function wphtc_page_script(){
			wp_enqueue_script("wpHtacControlJS", WP_PLUGIN_URL . '/wp-htaccess-control/wp-htaccess-control-ui.js');  
			}
		# Enqueue Ui Scripts on Plugin page
		function wphtc_page_style(){
			wp_enqueue_style("wpHtacControlCSS", WP_PLUGIN_URL . '/wp-htaccess-control/wp-htaccess-control-ui.css');  
			}
		# Options Page Actions
		function wphtc_page_action(){
			$action=$_POST['action']?$_POST['action']:$_GET['action'];
			global $echo;
			if(isset($action)){
				$WPhtc_data=get_option('WPhtc_data');
				switch($action){
					# if reseting everything just delete the option array
					case 'reset_rules':
						delete_option('WPhtc_data');
						$echo.=__('All rules reset.', 'wp-htaccess-control');
						global $wp_rewrite;
						$wp_rewrite->flush_rules();
						break;
					# if updating, save new rules in database and flush rewrite rules
					case 'update':	
						# get Rewrite Pairs
						for($i=0;$i<$_POST['WPhtc_num'];$i++){ 
							if(!empty($_POST['WPhtc_pat'.$i]) && !empty($_POST['WPhtc_sub'.$i])){
								$WPhtc_data['pats'][$i]=$_POST['WPhtc_pat'.$i];								
								$WPhtc_data['subs'][$i]=$_POST['WPhtc_sub'.$i];
								}					
							}
						if(count($WPhtc_data['pats'])>1 && $WPhtc_data['pats'][0]==''){
							unset($WPhtc_data['pats'][0],$WPhtc_data['subs'][0]);
							}
						if(is_array($WPhtc_data['pats'])&&is_array($WPhtc_data['subs'])){
							$WPhtc_data['pats']=array_values($WPhtc_data['pats']);
							$WPhtc_data['subs']=array_values($WPhtc_data['subs']);
							}
						# get Custom Htaccess
						$WPhtc_data['hta']=$_POST['WPhtc_hta'];
						# get Custom Author Permalink
						$WPhtc_data['cap']=$_POST['WPhtc_cap'];
						# get Custom Pagination Permalink
						$WPhtc_data['cpp']=$_POST['WPhtc_cpp'];
						# get htaccess Suggestions
						$WPhtc_data['disable_serversignature']=$_POST['WPhtc_disable_serversignature'];
						$WPhtc_data['disable_indexes']=$_POST['WPhtc_disable_indexes'];
						$WPhtc_data['up_limit']=(is_numeric($_POST['WPhtc_up_limit'])&&$_POST['WPhtc_up_limit']>0)?$_POST['WPhtc_up_limit']:'';
						$WPhtc_data['protect_wp_config']=$_POST['WPhtc_protect_wp_config'];
						$WPhtc_data['protect_htaccess']=$_POST['WPhtc_protect_htaccess'];
						$WPhtc_data['disable_hotlink']=trim($_POST['WPhtc_disable_hotlink']);
						$WPhtc_data['redirect_500']=trim($_POST['WPhtc_redirect_500']);
						$WPhtc_data['canon']=$_POST['WPhtc_canon'];
						$WPhtc_data['admin_email']=trim($_POST['WPhtc_admin_email']);
						$WPhtc_data['deflate']=trim($_POST['WPhtc_deflate']);
						$WPhtc_data['gzip']=trim($_POST['WPhtc_gzip']);
						
						# Unsetting inclusion of Author pages on Google XML Sitemap options, we'll add those links to the sitemap later with our Custom Author Permalink
						$WPhtc_data['sm_enabled']=$_POST['WPhtc_sm_enabled'];
						if($WPhtc_data['sm_enabled']==true){
							$SMoptions=get_option("sm_options");
							if($SMoptions&&is_array($SMoptions)){
								$SMoptions=get_option("sm_options");
								$SMoptions['sm_in_auth']=0;
								update_option('sm_options',$SMoptions);
								# Try to rebuild Sitemap
								do_action("sm_rebuild");
								}
							}
						# Update WP htaccess Control options
						update_option('WPhtc_data',$WPhtc_data);
						# Flush Rewrite Rules
						global $wp_rewrite;
						$wp_rewrite->flush_rules();
						$echo.=__('All options updated.', 'wp-htaccess-control');
						break;
					# deleting individual rewrite pair
					case 'delete_pair':
						if(isset($_GET['pair'])){
							# if it's the last pair
							if($_GET['pair']==0 && !isset($WPhtc_data['pats'][1])){
								unset($WPhtc_data['pats'], $WPhtc_data['subs']);
								$echo.=__('Wordpress Rewrite pairs reset.', 'wp-htaccess-control');
								}
							else{
								unset($WPhtc_data['pats'][$_GET['pair']],$WPhtc_data['subs'][$_GET['pair']]);
								$WPhtc_data['pats']=array_values($WPhtc_data['pats']);
								$WPhtc_data['subs']=array_values($WPhtc_data['subs']);
								$echo.=__('Wordpress Rewrite pair deleted.', 'wp-htaccess-control');
								}
							update_option('WPhtc_data',$WPhtc_data);
							global $wp_rewrite;
							$wp_rewrite->flush_rules();
							}
						break;
					}
				}
			}
		function formatted_print_r($data) {
				if(is_array($data)) { //If the given variable is an array, print using the print_r function.
					print "<pre>-----------------------\n";
					print_r($data);
					print "-----------------------</pre>";
				} elseif (is_object($data)) {
					print "<pre>==========================\n";
					var_dump($data);
					print "===========================</pre>";
				} else {
					print "=========&gt; ";
					var_dump($data);
					print " &lt;=========";
				}
			} 
		}
	} 
if (class_exists("WPhtc")) {
	$WPhtc = new WPhtc();
	}
if (isset($WPhtc)) {
	add_filter('mod_rewrite_rules', array($WPhtc,'wphtc_hta_rules'));
	add_filter('generate_rewrite_rules', array($WPhtc,'wphtc_wp_rewrite_rules'));
	add_filter('rewrite_rules_array',array($WPhtc,'wphtc_filter_rewrite_rules'));
	add_filter('author_link',array($WPhtc,'wphtc_filter_author_link'));
	add_filter('get_pagenum_link',array($WPhtc,'wphtc_filter_get_pagenum_link'));
	add_filter('redirect_canonical',array($WPhtc,'wphtc_filter_redirect_canonical'),10,4);
	add_action('admin_menu', array($WPhtc,'configure_menu'));
	add_action('sm_buildmap',array($WPhtc,'set_sm'));
	add_action('init', array($WPhtc,'wphtc_init'));
	}
?>